package Net::Async::xLM::API::Types::Results;
use v5.36.0;
use Object::Pad;

use Net::Async::xLM::API::Types;
use Object::PadX::Role::AutoMarshal;
use Object::PadX::Role::AutoJSON;
use Object::Pad::ClassAttr::Struct;

role Net::Async::xLM::API::Types::Results::Encoder::JSON {
  apply Net::Async::xLM::API::Types::Base;
  apply Object::PadX::Role::AutoJSON;
  apply Object::PadX::Role::AutoMarshal;

  use JSON::MaybeXS;
  my $_json = JSON::MaybeXS->new(utf8 => 1, convert_blessed => 1, canonical => 1);

  method _serialize() {
    my $json = $_json->encode($self);

    return $json;
  }

  method _content_type() {"application/json"}
  method _event_name() {"event"}
}

role Net::Async::xLM::API::Types::Results::Encoder::Raw {
  apply Net::Async::xLM::API::Types::Base;
  apply Object::PadX::Role::AutoJSON;
  apply Object::PadX::Role::AutoMarshal;

  use JSON::MaybeXS;

  method serialize() {
    ... # TODO this needs to give out bytes, how to decide that? meta programming?
  }
}

role Net::Async::xLM::API::Types::Results::Encoder::WWWForm {
  apply Net::Async::xLM::API::Types::Base;
  apply Object::PadX::Role::AutoJSON;
  apply Object::PadX::Role::AutoMarshal;

  use JSON::MaybeXS;

  method serialize() {
    ...
  }
}

class Net::Async::xLM::API::Types::Results::ToolCall :Struct {
  apply Net::Async::xLM::API::Types::Base;

  field $id :JSONStr = undef;
  field $type :JSONStr = undef; # always "function" right now, may get expanded in the future
  field $function :MarshalTo(Net::Async::xLM::API::Types::Results::FunctionCall) = undef;
}

class Net::Async::xLM::API::Types::Results::FunctionCall :Struct {
  apply Net::Async::xLM::API::Types::Base;

  field $arguments :JSONStr = undef; # TODO decode the json from this directly?
  field $name :JSONStr = undef;
}

class Net::Async::xLM::API::Types::Results::ChatMessage :Struct {
  apply Net::Async::xLM::API::Types::Base;

  field $content :JSONStr;
  field $tool_calls :MarshalTo([Net::Async::xLM::API::Types::Results::ToolCall]) = undef; # don't think my local server provides this
  field $role :JSONStr;
  field $function_call :MarshalTo(Net::Async::xLM::API::Types::Results::FunctionCall) = undef; # Depcrecated, might still happen
}

class Net::Async::xLM::API::Types::Results::ChatCompletionChoices :Struct {
  apply Net::Async::xLM::API::Types::Base;

  field $finish_reason :JSONStr;
  field $index :JSONNum;
  field $message :MarshalTo(Net::Async::xLM::API::Types::Results::ChatMessage);
}

class Net::Async::xLM::API::Types::Results::ChatCompletion :Struct {
  apply Net::Async::xLM::API::Types::Results::Encoder::JSON

  field $id :JSONStr;
  field $choices :MarshalTo([Net::Async::xLM::API::Types::Results::ChatCompletionChoices]);
  field $created :JSONStr;
  field $model :JSONStr;
  field $system_fingerprint :JSONStr = undef; # My local system doesn't provide this
  field $usage :MarshalTo(Net::Async::xLM::API::Types::Results::Usage);
  field $object :JSONStr;
}

class Net::Async::xLM::API::Types::Results::ChunkDelta :Struct {
  apply Net::Async::xLM::API::Types::Base;

  field $content :JSONStr;
  field $function_call :MarshalTo(Net::Async::xLM::API::Types::Results::FunctionCall) = undef;
  field $tool_cass :MarshalTo([Net::Async::xLM::API::Types::Results::ToolCall]) = undef;
  field $role :JSONStr;
}

class Net::Async::xLM::API::Types::Results::ChatCompletionChunkChoices :Struct {
  apply Net::Async::xLM::API::Types::Base;

  field $delta :MarshalTo(Net::Async::xLM::API::Types::Results::ChunkDelta);
  field $finish_reason :JSONStr;
  field $index :JSONStr;
}

# This is part of the streaming API
class Net::Async::xLM::API::Types::Results::ChatCompletionChunk :Struct {
  apply Net::Async::xLM::API::Types::Base;

  field $id :JSONStr;
  field $choices :MarshalTo(Net::Async::xLM::API::Types::Results::ChatCompletionChunkChoices);
  field $created :JSONStr;
  field $model :JSONStr;
  field $system_fingerprint :JSONStr = undef;
  field $object :JSONStr;
}

class Net::Async::xLM::API::Types::Results::Usage :Struct {
  apply Net::Async::xLM::API::Types::Base;

  field $total_tokens :JSONNum;
  field $prompt_tokens :JSONNum;
  field $completion_tokens :JSONNum = undef; # look at chat completions, is this the same
}

class Net::Async::xLM::API::Types::Results::LogProbs :Struct {
  apply Net::Async::xLM::API::Types::Base;

  # TODO what's the representation here?
  field $text_offset = undef;
  field $token_logprobs = undef;
  field $tokens = undef;
  field $top_logprobs = undef;
}

class Net::Async::xLM::API::Types::Results::CompletionChoices :Struct {
  apply Net::Async::xLM::API::Types::Base;

  field $text :JSONStr; 
  field $index :JSONNum;
  field $logprobs :MarshalTo(Net::Async::xLM::API::Types::Results::LogProbs) = undef; # TODO make nicer type?
  field $finish_reason :JSONStr = undef; # TODO enum? helper funcs for this class? ->is_finished?
}

class Net::Async::xLM::API::Types::Results::Completion :Struct {
  apply Net::Async::xLM::API::Types::Base;

  field $id :JSONStr;
  field $choices :MarshalTo([Net::Async::xLM::API::Types::Results::CompletionChoices]);
  field $created :JSONStr;
  field $model :JSONStr;
  field $system_fingerprint = undef; # my local implementation doesn't provide this, openai does it for tracking changes somehow
  field $usage :MarshalTo(Net::Async::xLM::API::Types::Results::Usage);
  field $object :JSONStr;
}


class Net::Async::xLM::API::Types::Results::Embedding :Struct {
  apply Net::Async::xLM::API::Types::Base;

  field $object :JSONStr;
  field $model :JSONStr;
  field $usage :MarshalTo(Net::Async::xLM::API::Types::Results::Usage);
  field $data :MarshalTo([Net::Async::xLM::API::Types::Results::EmbeddingData]);
}

class Net::Async::xLM::API::Types::Results::EmbeddingData :Struct {
  apply Net::Async::xLM::API::Types::Base;

  field $index :JSONNum;
  field $embedding :JSONList(JSONNum);
  field $object :JSONStr;
}

class Net::Async::xLM::API::Types::Results::ModelList :Struct {
  apply Net::Async::xLM::API::Types::Base;

  field $object :JSONStr = 'list';
  field $data :MarshalTo(Net::Async::xLM::API::Types::Results::ModelInfo);
}

class Net::Async::xLM::API::Types::Results::ModelInfo :Struct {
  apply Net::Async::xLM::API::Types::Base;

  field $created :JSONNum;
  field $id :JSONStr;
  field $object :JSONStr = "model";
  field $owned_by :JSONStr;
}

class Net::Async::xLM::API::Types::Results::Moderation :Struct {
  apply Net::Async::xLM::API::Types::Base;

  field $id :JSONStr;
  field $model :JSONStr;
  field $results :MarshalTo([Net::Async::xLM::API::Types::Results::ModerationResults]); # Not really sure why it's an array, the input doesn't allow multiple things to categorize
}

class Net::Async::xLM::API::Types::Results::ModerationResults :Struct {
  apply Net::Async::xLM::API::Types::Base;

  field $flagged :JSONBool;
  field $categories :MarshalTo(Net::Async::xLM::API::Types::Results::ModerationResultsCategories);
  field $category_scores :MarshalTo(Net::Async::xLM::API::Types::Results::ModerationResultsCategoryScores);
}

class Net::Async::xLM::API::Types::Results::ModerationResultsCategories :Struct {
  apply Net::Async::xLM::API::Types::Base;

  field $hate :JSONBool;
  field $hate_threatening :JSONBool :JSONKey(hate/threatening);
  field $harassment :JSONBool;
  field $harassment_threatening :JSONBool :JSONKey(harassment/threatening);
  field $self_harm :JSONBool :JSONKey(self-harm);
  field $self_harm_intent :JSONBool :JSONKey(self-harm/intent);
  field $self_harm_instructions :JSONBool :JSONKey(self-harm/instructions);
  field $sexual :JSONBool;
  field $sexual_minors :JSONBool :JSONKey(sexual/minors);
  field $violence :JSONBool;
  field $violence_graphic :JSONBool :JSONKey(violence/graphic);
}

class Net::Async::xLM::API::Types::Results::ModerationResultsCategoryScores :Struct {
  apply Net::Async::xLM::API::Types::Base;

  field $hate :JSONNum;
  field $hate_threatening :JSONNum :JSONKey(hate/threatening);
  field $harassment :JSONNum;
  field $harassment_threatening :JSONNum :JSONKey(harassment/threatening);
  field $self_harm :JSONNum :JSONKey(self-harm);
  field $self_harm_intent :JSONNum :JSONKey(self-harm/intent);
  field $self_harm_instructions :JSONNum :JSONKey(self-harm/instructions);
  field $sexual :JSONNum;
  field $sexual_minors :JSONNum :JSONKey(sexual/minors);
  field $violence :JSONNum;
  field $violence_graphic :JSONNum :JSONKey(violence/graphic);
}

class Net::Async::xLM::API::Types::Results::Image :Struct {
  apply Net::Async::xLM::API::Types::Base;

  field $b64_json :JSONStr = undef;
  field $url :JSONStr = undef;
  field $revised_prompt :JSONStr = undef;

  ADJUST {
    die "Missing required value one of b64_json or url" unless ($b64_json or $url);
  }
}
